/*
Pin Used:
Digital: 2,3,4,5,6,7   LCD
         13            Led
         9,10         Step pin9, Dir pin10,
         2,3           Encoder
Analog:  A0            Buttons
*/

// ***** Libaries included *****
#include <Wire.h>
#include <LiquidCrystal.h>
#include <buttons.h>
#include <MENWIZ.h>
#include <EEPROM.h>
#include <AccelStepper.h>

// Create global object for menu and lcd
menwiz menu;
LiquidCrystal lcd( 2, 3, 4, 5, 6, 7);

//***** Stepper Motor Parameters *****
AccelStepper stepper1(1, 9, 10); // Step pin9, Dir pin10;

 const int stepsPerRevolution = 200;


boolean wr=0;                         // Instantiate global variables to bind to menu
const int buttonPin = A5;             // 4 button switch circuit input connected to analog pin 0
const int led13Pin = 13;              // the number of the LED output pin
int stepCount = 0;                    // number of steps the motor has taken
int HeightZ = 0;                      //ABS Height Variable = mm
int ActualHeightZ = 0;
int NewHeightZ;
int MoveREL;                          //Rel Move Variable = mm
int Height2;
boolean buttonBlock = 0;
boolean buttonAct = 0;                // flag 1
boolean stopMenu = 0;                 // flag 2
byte buttonPressed = 0;               // which button was pressed
byte lastButtonPressed = 0;           // prev button pressed
int buttonValue = 0;                  // read value from analog port for the 4 navigation pushbuttons
long menuOffTime = 0;                 //used for timer

extern byte MW_navbtn;                //needed to run the software key navigation in Menwiz


void setup()
 {
 Serial.begin(9600);          // Output to serial writing
  digitalWrite((A0), HIGH);        // enable the 20k internal pullup for UNO based board for analog port A0

 //****** Motor Speed & Accel ******
  stepper1.setMaxSpeed(900);
      stepper1.setSpeed(400);
   stepper1.setAcceleration(200);
   //stepper1.setMinPulseWidth(20);
 //********** Menu and LCD *********
  char b[84];
  _menu *r,*s1,*s2,*s3;

 // initialize the menu object (20x4 LCD)
  menu.begin(&lcd,20,4);           //initialize the menwiz menu object passing the lcd object and the colums and rows params 
  menu.addUsrNav(navMenu,4);
  MW_navbtn=4;                     // force 4 or 6 buttons mode for menu navigation -> MW_navbtn=4; or MW_navbtn=6;

 //****** create the menu tree *****
  r=menu.addMenu(MW_ROOT,NULL,F("ROTALA RT"));          // create a root menu at first (required)

 //****** Router Table Fence ******
  s1=menu.addMenu(MW_SUBMENU,r,F("Fence"));               // add a submenu node 2 to the root menu (Control the Distance of The Fence)
   s2=menu.addMenu(MW_SUBMENU,s1,F("Fence Distance"));    // (mm Fence Distance From  The Mill)
      s3=menu.addMenu(MW_VAR,s2,F("Absolute Distance"));  // add a terminal node in the menu tree (that is "variable")
           s3->addVar(MW_AUTO_INT,&HeightZ,0,100,1);      // int type, fd binded variable, rage 0-100, step 1
       s3=menu.addMenu(MW_VAR,s2,F("Move Fence!!"));      // Call the Action ABS_Height to move the Stepper in ABS mode
           s3->addVar(MW_ACTION,ABS_Distance);            // Execute the Action Above
   s2=menu.addMenu(MW_SUBMENU,s1,F("Fence Move"));        // add a terminal node in the menu tree (that is "variable")
      s3=menu.addMenu(MW_VAR,s2,F("Relative Move"));      // (Move OF a certain mm)
          s3->addVar(MW_AUTO_INT,&MoveREL,-100,100,1);    // int type, fd binded variable, rage -100 +100, step 1
      s3=menu.addMenu(MW_VAR,s2,F("Move Fence!!"));       // Call the Action moveSAW to move the Stepper in REL mode
           s3->addVar(MW_ACTION,MoveFENCE);               // Execute the Action Above
   s2=menu.addMenu(MW_VAR,s1,F("Zero"));                  //add a terminal node in the menu tree (that is "variable"); (should set the zero)
   s2=menu.addMenu(MW_VAR,s1,F("Setup"));                 //add a terminal node in the menu tree (that is "variable");  (set the parameters)
 
  
 //******* Router Table Mill Height *******
   s1=menu.addMenu(MW_SUBMENU,r,F("Mill Height"));                    //add a submenu node 2 to the root menu (control the heigh of my Mill)
/*  s2=menu.addMenu(MW_VAR,s1,F("ABS Height"));                         //add a terminal node in the menu tree (that is "variable"); (should move at a certain mm height)
     s2->addVar(MW_AUTO_INT,&Height2,0,100,1);     //int type, fd binded variable, rage 0-100, step 1
      s2=menu.addMenu(MW_VAR,s2,F("moveMOTOR2"));
       s2->addVar(MW_ACTION,moveMOTOR2);
    s2=menu.addMenu(MW_VAR,s1,F("REL Move"));                         //add a terminal node in the menu tree (that is "variable"); (should move of a certain mm the height)
     s2->addVar(MW_AUTO_INT,&Height1,-100,100,1);                   //int type, fd binded variable, rage 0-100, step 1
      s2=menu.addMenu(MW_VAR,s1,F("moveMOTOR"));
       s2->addVar(MW_ACTION,moveMOTOR);
    s2=menu.addMenu(MW_VAR,s1,F("Zero"));                         //add a terminal node in the menu tree (that is "variable"); (should set the zero)
    s2=menu.addMenu(MW_VAR,s1,F("Setup"));                        //add a terminal node in the menu tree (that is "variable");  (set the parameters)
 */ 

  //******* Router Inclination *******
  s1=menu.addMenu(MW_SUBMENU,r,F("Router Inclination"));                    //add a submenu node 2 to the root menu (control the heigh of my Planner)
   s2=menu.addMenu(MW_SUBMENU,s1,F("Inclinatin Deg"));                         //add a terminal node in the menu tree (that is "variable"); (should move at a certain mm height)
    s3=menu.addMenu(MW_VAR,s2,F("Absolutes Deg"));
     s3->addVar(MW_AUTO_INT,&HeightZ,0,100,1);     //int type, fd binded variable, rage 0-100, step 1
    s3=menu.addMenu(MW_VAR,s2,F("Test"));
     s3->addVar(MW_ACTION,Test);
   s2=menu.addMenu(MW_VAR,s1,F("REL Move"));                         //add a terminal node in the menu tree (that is "variable"); (should move of a certain mm the height)
     s2->addVar(MW_AUTO_INT,&Height2,-100,100,1);                   //int type, fd binded variable, rage 0-100, step 1
    s2=menu.addMenu(MW_VAR,s1,F("moveMOTOR2"));
     s2->addVar(MW_ACTION,Test);
  
  /*
  //++++++ Splash screen +++++++
  sprintf(menu.sbuf,"TEST minimum menu\nwith 4 or 6\nanalog push buttons\n",1);
  menu.addSplash((char *) menu.sbuf, 1000);

  //++++++ Userscreen ++++++++++
  // create an user define screen callback to activate after X secs since last button push
  menu.addUsrScreen(msc,3000);
  */
  pinMode(led13Pin, OUTPUT);                // initialize the led as a output control pin
}

void loop()
 {

  menu.draw();     // NAVIGATION MANAGEMENT & DRAWING ON LCD.Has to be the first
  ReadButtons();
        stepper1.run();
  /*
  put your regular code here
   */
 }


void ABS_Distance()                   // Absolute Fence Distance Function
 {
  NewHeightZ = (HeightZ - ActualHeightZ);
  stepper1.moveTo(NewHeightZ*800);  // move Height1 mm's
   lcd.setCursor (0,2);
    lcd.print ("Move mm ");
     lcd.print (NewHeightZ);
  lcd.setCursor (0,3);
   lcd.print ("Distance mm ");
    lcd.print (HeightZ);
   delay (2000);
  ActualHeightZ = HeightZ;
 }


void MoveFENCE()                      // This is the Relative Movement of the SAW. WORKING ! :-)
 {
  stepper1.moveTo(MoveREL*800);    // move the Fence X mm's
  lcd.setCursor (0,2);
   lcd.print ("Move mm ");
    lcd.print (MoveREL);
//     lcd.print (" mm");
//      delay (5000);
   HeightZ = (HeightZ+MoveREL);
    ActualHeightZ = HeightZ;
  lcd.setCursor (0,3);
   lcd.print ("Distance mm ");
    lcd.print (HeightZ);
//     lcd.print (" mm");
      delay (5000);
 }


void Test()
 {
  stepper1.moveTo(200); // move Height2 mm's
   lcd.print ("Alt Fresa: mm ");
   lcd.print (Height2);
  delay (5000);
 }

// Here is the working coce to use MENWIZ with 4 Analog Pushbottons.
//  ++++++ functions +++++++

void ReadButtons()

// ++++ Control 4 buttons ++++
 {
  lastButtonPressed = buttonPressed;
  buttonValue = analogRead(buttonPin); //analog value to simulate the pushed buttons

   if(buttonValue >= 850)
    {
     buttonPressed = 0;
     noButtonPressed();               // ? calling an extra fucntion
    }
   else  if(buttonValue >= 380 & buttonValue <= 550)
    {
     buttonPressed = 4;
     buttonAct = 1;                         // set the menu flag1
    }
   else if(buttonValue >= 200 & buttonValue <=350)
    {    
     buttonPressed = 3;
     buttonAct = 1;                         // set the menu flag1
    }
   else if(buttonValue >= 0 & buttonValue <=50)
    {    
     buttonPressed = 2;
     buttonAct = 1;                         // set the menu flag1
    }
   else if(buttonValue >= 80 & buttonValue <=200)
    {    
     buttonPressed = 1;
     buttonAct = 1;                         // set the menu flag1
    }
 }

int noButtonPressed()
 {
  return MW_BTNULL;
 }

int navMenu()                              // called from menu.draw
 {
  /*
   As soon as a button is pushed the first time flag 1 is set to HIGH and if the buttonnumber is not 0 then a timer is started.
   The menu action then should only be performed once.
   After 400 msecs the flag will be set to LOW and a new buttonpush can be processed.
   Menu action is blocked for 400 msec even if the same button is being kept pressed for 2000 msecs by flag2.
   */
  long menublockTime = millis();

  if (buttonAct == 1 && buttonPressed != 0 && stopMenu == 0)  // we have a state were we process a menu navigation step once and block it for 2 secs
   {
    digitalWrite(led13Pin,HIGH);                              // set timer led ON
    menuOffTime = menublockTime  + 400;                       //start the timer. You can change this blocking time but do not make it lower aa 200 msecs
    stopMenu = 1;

    switch (buttonPressed)
    {
    case 1: // Up
      return MW_BTU;
      break;
    case 2: // Confirm
      return MW_BTC;
      break;
    case 3: // Down
      return MW_BTD;
      break;    
    case 4: // Escape
      return MW_BTE;
      break;
    }
   }

  if (menuOffTime != 0  &&  menublockTime  > menuOffTime)     //  Reset of the timer so a new menu action can be processed
   {
    buttonAct = 0; // resets the flag 1
    buttonPressed = 0;
    menuOffTime = 0;  // resets timer to zero
    stopMenu = 0;
    digitalWrite(led13Pin,LOW);  // set timer led OFF
   }
 }

/*
// user defined default screen.
void msc()
{
  sprintf(menu.sbuf,"User screen\nUptime (s): %ld\nFree mem  : %d\n\n",millis()/1000,(int)menu.freeRam());
  menu.drawUsrScreen(menu.sbuf);
}*/
